@@ -83,7 +83,7 @@ gconf_key_directory  (const gchar* key)
 
   if (end == NULL)
     {
-      gconf_log(GCL_ERR, _("No '/' in key `%s'"), key);
+      gconf_log(GCL_ERR, _("No '/' in key \"%s\""), key);
       return NULL;
     }
 
@@ -294,8 +294,8 @@ gconf_value_from_corba_value(const ConfigValue* value)
 }
 
 void          
-fill_corba_value_from_gconf_value(const GConfValue* value, 
-                                   ConfigValue* cv)
+gconf_fill_corba_value_from_gconf_value(const GConfValue *value, 
+                                        ConfigValue      *cv)
 {
   if (value == NULL)
     {
@@ -323,8 +323,8 @@ fill_corba_value_from_gconf_value(const GConfValue* value,
       break;
     case GCONF_VALUE_SCHEMA:
       cv->_d = SchemaVal;
-      fill_corba_schema_from_gconf_schema(gconf_value_get_schema(value),
-                                           &cv->_u.schema_value);
+      gconf_fill_corba_schema_from_gconf_schema (gconf_value_get_schema(value),
+                                                 &cv->_u.schema_value);
       break;
     case GCONF_VALUE_LIST:
       {
@@ -375,8 +375,8 @@ fill_corba_value_from_gconf_value(const GConfValue* value,
         while (list != NULL)
           {
             /* That dubious ConfigBasicValue->ConfigValue cast again */
-            fill_corba_value_from_gconf_value((GConfValue*)list->data,
-                                               (ConfigValue*)&cv->_u.list_value.seq._buffer[i]);
+            gconf_fill_corba_value_from_gconf_value((GConfValue*)list->data,
+                                                    (ConfigValue*)&cv->_u.list_value.seq._buffer[i]);
 
             list = g_slist_next(list);
             ++i;
@@ -394,10 +394,10 @@ fill_corba_value_from_gconf_value(const GConfValue* value,
         CORBA_sequence_set_release(&cv->_u.pair_value, TRUE);
         
         /* dubious cast */
-        fill_corba_value_from_gconf_value(gconf_value_get_car(value),
-                                           (ConfigValue*)&cv->_u.pair_value._buffer[0]);
-        fill_corba_value_from_gconf_value(gconf_value_get_cdr(value),
-                                           (ConfigValue*)&cv->_u.pair_value._buffer[1]);
+        gconf_fill_corba_value_from_gconf_value (gconf_value_get_car(value),
+                                                 (ConfigValue*)&cv->_u.pair_value._buffer[0]);
+        gconf_fill_corba_value_from_gconf_value(gconf_value_get_cdr(value),
+                                                (ConfigValue*)&cv->_u.pair_value._buffer[1]);
       }
       break;
       
@@ -412,19 +412,19 @@ fill_corba_value_from_gconf_value(const GConfValue* value,
 }
 
 ConfigValue*  
-corba_value_from_gconf_value(const GConfValue* value)
+gconf_corba_value_from_gconf_value (const GConfValue* value)
 {
   ConfigValue* cv;
 
   cv = ConfigValue__alloc();
 
-  fill_corba_value_from_gconf_value(value, cv);
+  gconf_fill_corba_value_from_gconf_value(value, cv);
 
   return cv;
 }
 
 ConfigValue*  
-invalid_corba_value()
+gconf_invalid_corba_value ()
 {
   ConfigValue* cv;
 
@@ -518,48 +518,48 @@ gconf_type_from_corba_type(ConfigValueType type)
 }
 
 void          
-fill_corba_schema_from_gconf_schema(const GConfSchema* sc, 
-                                     ConfigSchema* cs)
+gconf_fill_corba_schema_from_gconf_schema(const GConfSchema *sc, 
+                                          ConfigSchema      *cs)
 {
-  cs->value_type = corba_type_from_gconf_type(sc->type);
-  cs->value_list_type = corba_type_from_gconf_type(sc->list_type);
-  cs->value_car_type = corba_type_from_gconf_type(sc->car_type);
-  cs->value_cdr_type = corba_type_from_gconf_type(sc->cdr_type);
+  cs->value_type = corba_type_from_gconf_type (gconf_schema_get_type (sc));
+  cs->value_list_type = corba_type_from_gconf_type (gconf_schema_get_list_type (sc));
+  cs->value_car_type = corba_type_from_gconf_type (gconf_schema_get_car_type (sc));
+  cs->value_cdr_type = corba_type_from_gconf_type (gconf_schema_get_cdr_type (sc));
 
-  cs->locale = CORBA_string_dup(sc->locale ? sc->locale : "");
-  cs->short_desc = CORBA_string_dup(sc->short_desc ? sc->short_desc : "");
-  cs->long_desc = CORBA_string_dup(sc->long_desc ? sc->long_desc : "");
-  cs->owner = CORBA_string_dup(sc->owner ? sc->owner : "");
+  cs->locale = CORBA_string_dup (gconf_schema_get_locale (sc) ? gconf_schema_get_locale (sc) : "");
+  cs->short_desc = CORBA_string_dup (gconf_schema_get_short_desc (sc) ? gconf_schema_get_short_desc (sc) : "");
+  cs->long_desc = CORBA_string_dup (gconf_schema_get_long_desc (sc) ? gconf_schema_get_long_desc (sc) : "");
+  cs->owner = CORBA_string_dup (gconf_schema_get_owner (sc) ? gconf_schema_get_owner (sc) : "");
 
   {
     gchar* encoded;
     GConfValue* default_val;
 
-    default_val = gconf_schema_get_default_value(sc);
+    default_val = gconf_schema_get_default_value (sc);
 
     if (default_val)
       {
-        encoded = gconf_value_encode(default_val);
+        encoded = gconf_value_encode (default_val);
 
-        g_assert(encoded != NULL);
+        g_assert (encoded != NULL);
 
-        cs->encoded_default_value = CORBA_string_dup(encoded);
+        cs->encoded_default_value = CORBA_string_dup (encoded);
 
-        g_free(encoded);
+        g_free (encoded);
       }
     else
-      cs->encoded_default_value = CORBA_string_dup("");
+      cs->encoded_default_value = CORBA_string_dup ("");
   }
 }
 
 ConfigSchema* 
-corba_schema_from_gconf_schema(const GConfSchema* sc)
+gconf_corba_schema_from_gconf_schema (const GConfSchema* sc)
 {
   ConfigSchema* cs;
 
-  cs = ConfigSchema__alloc();
+  cs = ConfigSchema__alloc ();
 
-  fill_corba_schema_from_gconf_schema(sc, cs);
+  gconf_fill_corba_schema_from_gconf_schema (sc, cs);
 
   return cs;
 }
@@ -696,7 +696,7 @@ unquote_string(gchar* s)
   gchar* end;
 
   /* Strip whitespace and first quote from front of string */
-  while (*s && (isspace(*s) || (*s == '"')))
+  while (*s && (g_ascii_isspace(*s) || (*s == '"')))
     ++s;
 
   end = s;
@@ -706,7 +706,7 @@ unquote_string(gchar* s)
   --end; /* one back from '\0' */
 
   /* Strip whitespace and last quote from end of string */
-  while ((end > s) && (isspace(*end) || (*end == '"')))
+  while ((end > s) && (g_ascii_isspace(*end) || (*end == '"')))
     {
       *end = '\0';
       --end;
@@ -845,7 +845,7 @@ gconf_load_source_path(const gchar* filename, GError** err)
     {
       gchar* s = buf;
       
-      while (*s && isspace(*s))
+      while (*s && g_ascii_isspace(*s))
         ++s;
 
       if (*s == '#')
@@ -863,7 +863,7 @@ gconf_load_source_path(const gchar* filename, GError** err)
           gchar *varsubst;
           
           s += 7;
-          while (isspace(*s))
+          while (g_ascii_isspace(*s))
             s++;
           unq = unquote_string(s);
 
@@ -977,46 +977,30 @@ gconf_string_to_gulong(const gchar* str)
 }
 
 gboolean
-gconf_string_to_double(const gchar* str, gdouble* retloc)
+gconf_string_to_double(const gchar* str,
+                       gdouble*     retloc)
 {
-  int res;
-  char *old_locale;
+  char *end;
 
-  /* make sure we write values to files in a consistent manner */
-  old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-  setlocale (LC_NUMERIC, "C");
-
-  *retloc = 0.0;
-  res = sscanf (str, "%lf", retloc);
-
-  setlocale (LC_NUMERIC, old_locale);
-  g_free (old_locale);
-
-  if (res == 1)
-    return TRUE;
+  errno = 0;
+  *retloc = g_ascii_strtod (str, &end);
+  if (end == str || errno != 0)
+    {
+      *retloc = 0.0;
+      return FALSE;
+    }
   else
-    return FALSE;
+    return TRUE;
 }
 
 gchar*
-gconf_double_to_string(gdouble val)
+gconf_double_to_string (gdouble val)
 {
-  char str[101 + DBL_DIG];
-  char *old_locale;
+  char str[G_ASCII_DTOSTR_BUF_SIZE];
 
-  /* make sure we write values to files in a consistent manner */
-  old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-  setlocale (LC_NUMERIC, "C");
+  g_ascii_dtostr (str, G_ASCII_DTOSTR_BUF_SIZE, val);
   
-  if (fabs (val) < 1e9 && fabs (val) > 1e-5)
-    g_snprintf (str, 100 + DBL_DIG, "%.*g", DBL_DIG, val);
-  else
-    g_snprintf (str, 100 + DBL_DIG, "%f", val);
-
-  setlocale (LC_NUMERIC, old_locale);
-  g_free (old_locale);
-  
-  return g_strdup(str);
+  return g_strdup (str);
 }
 
 const gchar*
@@ -1378,13 +1362,9 @@ gconf_value_list_to_primitive_list_destructive(GConfValue* val,
 
   g_assert(gconf_value_get_list_type(val) == list_type);
       
-  retval = gconf_value_get_list(val);
-
-  /* Cheating the API to avoid a list copy; set this to NULL to
-         avoid destroying the list */
-  val->d.list_data.list = NULL;
+  retval = gconf_value_steal_list (val);
       
-  gconf_value_free(val);
+  gconf_value_free (val);
   val = NULL;
       
   {
@@ -1403,9 +1383,12 @@ gconf_value_list_to_primitive_list_destructive(GConfValue* val,
         switch (list_type)
           {
           case GCONF_VALUE_INT:
-          case GCONF_VALUE_BOOL:
             tmp->data = GINT_TO_POINTER(gconf_value_get_int(elem));
             break;
+
+          case GCONF_VALUE_BOOL:
+            tmp->data = GINT_TO_POINTER(gconf_value_get_bool(elem));
+            break;
                 
           case GCONF_VALUE_FLOAT:
             {
@@ -1416,19 +1399,11 @@ gconf_value_list_to_primitive_list_destructive(GConfValue* val,
             break;
 
           case GCONF_VALUE_STRING:
-            {
-              /* Cheat again, and steal the string from the value */
-              tmp->data = elem->d.string_data;
-              elem->d.string_data = NULL;
-            }
+            tmp->data = gconf_value_steal_string (elem);
             break;
 
           case GCONF_VALUE_SCHEMA:
-            {
-              /* and also steal the schema... */
-              tmp->data = elem->d.schema_data;
-              elem->d.schema_data = NULL;
-            }
+            tmp->data = gconf_value_steal_schema (elem);
             break;
                 
           default:
@@ -1462,9 +1437,7 @@ primitive_value(gpointer retloc, GConfValue* val)
 
     case GCONF_VALUE_STRING:
       {
-        *((gchar**)retloc) = val->d.string_data;
-        /* cheat and steal the string to avoid a copy */
-        val->d.string_data = NULL;
+        *((gchar**)retloc) = gconf_value_steal_string (val);
       }
       break;
 
@@ -1473,7 +1446,7 @@ primitive_value(gpointer retloc, GConfValue* val)
       break;
 
     case GCONF_VALUE_SCHEMA:
-      *((GConfSchema**)retloc) = gconf_value_get_schema(val);
+      *((GConfSchema**)retloc) = gconf_value_steal_schema(val);
       break;
       
     default:
@@ -1795,7 +1768,7 @@ static gchar type_byte(GConfValueType type)
     }
 }
 
-GConfValueType
+static GConfValueType
 byte_type(gchar byte)
 {
   switch (byte)
@@ -2401,6 +2374,7 @@ open_empty_locked_file (const gchar *directory,
     }
 
   /* We have the lock on filename, so delete it */
+  /* FIXME this leaves .nfs32423432 cruft */
   unlink (filename);
   close (fd);
   fd = -1;
@@ -2412,8 +2386,8 @@ open_empty_locked_file (const gchar *directory,
 }
 
 static ConfigServer
-read_current_server (const gchar *iorfile,
-                     gboolean     warn_if_fail)
+read_current_server_and_set_warning (const gchar *iorfile,
+                                     GString     *warning)
 {
   FILE *fp;
   
@@ -2421,9 +2395,10 @@ read_current_server (const gchar *iorfile,
           
   if (fp == NULL)
     {
-      if (warn_if_fail)
-        gconf_log (GCL_WARNING, _("IOR file '%s' not opened successfully, no gconfd located: %s"),
-                   iorfile, g_strerror (errno));
+      if (warning)
+        g_string_append_printf (warning,
+                                _("IOR file '%s' not opened successfully, no gconfd located: %s"),
+                                iorfile, g_strerror (errno));
 
       return CORBA_OBJECT_NIL;
     }
@@ -2450,10 +2425,10 @@ read_current_server (const gchar *iorfile,
           str[2] == 'n' &&
           str[3] == 'e')
         {
-          if (warn_if_fail)
-            gconf_log (GCL_WARNING,
-                       _("gconftool or other non-gconfd process has the lock file '%s'"),
-                       iorfile);          
+          if (warning)
+            g_string_append_printf (warning,
+                                    _("gconftool or other non-gconfd process has the lock file '%s'"),
+                                    iorfile); 
         }
       else /* file contains daemon IOR */
         {
@@ -2467,9 +2442,9 @@ read_current_server (const gchar *iorfile,
 
           if (orb == NULL)
             {
-              if (warn_if_fail)
-                gconf_log (GCL_WARNING,
-                           _("couldn't contact ORB to resolve existing gconfd object reference"));
+              if (warning)
+                g_string_append_printf (warning,
+                                        _("couldn't contact ORB to resolve existing gconfd object reference"));
               return CORBA_OBJECT_NIL;
             }
                   
@@ -2477,10 +2452,10 @@ read_current_server (const gchar *iorfile,
           CORBA_exception_free (&ev);
 
           if (server == CORBA_OBJECT_NIL &&
-              warn_if_fail)
-            gconf_log (GCL_WARNING,
-                       _("Failed to convert IOR '%s' to an object reference"),
-                       str);
+              warning)
+            g_string_append_printf (warning,
+                                    _("Failed to convert IOR '%s' to an object reference"),
+                                    str);
           
           return server;
         }
@@ -2489,6 +2464,28 @@ read_current_server (const gchar *iorfile,
     }
 }
 
+static ConfigServer
+read_current_server (const gchar *iorfile,
+                     gboolean     warn_if_fail)
+{
+  GString *warning;
+  ConfigServer server;
+  
+  if (warn_if_fail)
+    warning = g_string_new (NULL);
+  else
+    warning = NULL;
+
+  server = read_current_server_and_set_warning (iorfile, warning);
+
+  if (warning->len > 0)
+    gconf_log (GCL_WARNING, "%s", warning->str);
+
+  g_string_free (warning, TRUE);
+
+  return server;
+}
+
 GConfLock*
 gconf_get_lock_or_current_holder (const gchar  *lock_directory,
                                   ConfigServer *current_server,
@@ -2692,93 +2689,67 @@ gconf_release_lock (GConfLock *lock,
  * of the sort; it just reads it. It does do the object_to_string
  */
 ConfigServer
-gconf_get_current_lock_holder  (const gchar *lock_directory)
+gconf_get_current_lock_holder  (const gchar *lock_directory,
+                                GString     *failure_log)
 {
   char *iorfile;
   ConfigServer server;
 
   iorfile = g_strconcat (lock_directory, "/ior", NULL);
-  server = read_current_server (iorfile, FALSE);
+  server = read_current_server_and_set_warning (iorfile, failure_log);
   g_free (iorfile);
   return server;
 }
 
-#include <netinet/in.h>
-#include <netdb.h>
-#include <arpa/inet.h>
-const char *
-get_hostname (void)
-{
-	static char *hostname = NULL;
-	char hn_tmp[65];
-	gulong ha_tmp;
-	struct hostent *hent;
-
-	if (!hostname) {
-		gethostname (hn_tmp, sizeof (hn_tmp) - 1);
-
-		hent = gethostbyname (hn_tmp);
-		if (hent) {
-			memcpy (&ha_tmp, hent->h_addr, 4);
-			hent = gethostbyaddr ((char *)&ha_tmp, 4, AF_INET);
-			if (hent)
-				hostname = g_strdup (hent->h_name);
-			else
-				hostname =
-					g_strdup (inet_ntoa
-						  (*
-						   ((struct in_addr *)
-						    &ha_tmp)));
-		} else
-			hostname = g_strdup (hn_tmp);
-	}
-
-	return hostname;
-}
+static CORBA_ORB gconf_orb = CORBA_OBJECT_NIL;      
 
 CORBA_ORB
 gconf_orb_get (void)
 {
-  if (TRUE || gconf_in_daemon_mode ())
+  if (gconf_orb == CORBA_OBJECT_NIL)
     {
-      static CORBA_ORB gconf_orb = CORBA_OBJECT_NIL;      
+      CORBA_Environment ev;
+      int argc = 1;
+      char *argv[] = { "gconf", NULL };
 
-      if (gconf_orb == CORBA_OBJECT_NIL)
+      _gconf_init_i18n ();
+      
+      CORBA_exception_init (&ev);
+      
+      gconf_orb = CORBA_ORB_init (&argc, argv, "orbit-local-orb", &ev);
+      g_assert (ev._major == CORBA_NO_EXCEPTION);
+
+      CORBA_exception_free (&ev);
+    }
+
+  return gconf_orb;
+}
+
+int
+gconf_orb_release (void)
+{
+  int ret = 0;
+
+  if (gconf_orb != CORBA_OBJECT_NIL)
+    {
+      CORBA_ORB orb = gconf_orb;
+      CORBA_Environment ev;
+
+      gconf_orb = CORBA_OBJECT_NIL;
+
+      CORBA_exception_init (&ev);
+
+      CORBA_ORB_destroy (orb, &ev);
+      CORBA_Object_release ((CORBA_Object)orb, &ev);
+
+      if (ev._major != CORBA_NO_EXCEPTION)
         {
-          CORBA_Environment ev;
-          int argc = 1;
-          char *argv[] = { "gconf", NULL };
-          CORBA_Context context;
-          const char *hostname;
-      
-          CORBA_exception_init (&ev);
-      
-          gconf_orb = CORBA_ORB_init (&argc, argv, "orbit-local-orb", &ev);
-          g_assert (ev._major == CORBA_NO_EXCEPTION);
-
-          /* Set values in default context */
-          CORBA_ORB_get_default_context (gconf_orb, &context, &ev);
-          g_assert (ev._major == CORBA_NO_EXCEPTION);
-          
-          hostname = get_hostname ();
-          CORBA_Context_set_one_value (context, "hostname",
-                                       (char *) hostname, &ev);
-          CORBA_Context_set_one_value (context, "domain", "user", &ev);
-          CORBA_Context_set_one_value (context, "username",
-                                       g_get_user_name (), &ev);
-          
-          CORBA_exception_free (&ev);
+          ret = 1;
         }
-      
-      return gconf_orb;
-    }
-  else
-    {
-      /* FIXME do we have to do this if an app is using bonobo activation? */
-#if 0
-      return bonobo_activation_orb_get ();
-#endif
+      CORBA_exception_free (&ev);
     }
+
+  return ret;
 }
 
 char*
@@ -2834,7 +2805,10 @@ gconf_activate_server (gboolean  start_if_not_found,
   char *argv[3];
   char *gconfd_dir;
   char *lock_dir;
+  GString *failure_log;
   CORBA_Environment ev;
+
+  failure_log = g_string_new (NULL);
   
   gconfd_dir = gconf_get_daemon_dir ();
   
@@ -2843,9 +2817,10 @@ gconf_activate_server (gboolean  start_if_not_found,
                gconfd_dir, g_strerror (errno));
 
   g_free (gconfd_dir);
-  
+
+  g_string_append (failure_log, " 1: ");
   lock_dir = gconf_get_lock_dir ();
-  server = gconf_get_current_lock_holder (lock_dir);
+  server = gconf_get_current_lock_holder (lock_dir, failure_log);
   g_free (lock_dir);
 
   /* Confirm server exists */
@@ -2856,13 +2831,22 @@ gconf_activate_server (gboolean  start_if_not_found,
       ConfigServer_ping (server, &ev);
       
       if (ev._major != CORBA_NO_EXCEPTION)
-        server = CORBA_OBJECT_NIL;
+        {
+          server = CORBA_OBJECT_NIL;
+
+          g_string_append_printf (failure_log,
+                                  _("Server ping error: %s"),
+                                  CORBA_exception_id (&ev));
+        }
     }
 
   CORBA_exception_free (&ev);
   
   if (server != CORBA_OBJECT_NIL)
-    return server;
+    {
+      g_string_free (failure_log, TRUE);
+      return server;
+    }
   
   if (start_if_not_found)
     {
@@ -2877,7 +2861,7 @@ gconf_activate_server (gboolean  start_if_not_found,
           goto out;
         }
 
-      argv[0] = g_strconcat (GCONF_BINDIR, "/" GCONFD, NULL);
+      argv[0] = g_strconcat (GCONF_SERVERDIR, "/" GCONFD, NULL);
       argv[1] = g_strdup_printf ("%d", p[1]);
       argv[2] = NULL;
   
@@ -2908,8 +2892,9 @@ gconf_activate_server (gboolean  start_if_not_found,
       /* Block until server starts up */
       read (p[0], buf, 1);
 
+      g_string_append (failure_log, " 2: ");
       lock_dir = gconf_get_lock_dir ();
-      server = gconf_get_current_lock_holder (lock_dir);
+      server = gconf_get_current_lock_holder (lock_dir, failure_log);
       g_free (lock_dir);
     }
   
@@ -2920,10 +2905,57 @@ gconf_activate_server (gboolean  start_if_not_found,
     g_set_error (error,
                  GCONF_ERROR,
                  GCONF_ERROR_NO_SERVER,
-                 _("Failed to contact configuration server (a likely cause of this is that you have an existing configuration server (gconfd) running, but it isn't reachable from here - if you're logged in from two machines at once, you may need to enable TCP networking for ORBit)\n"));
+                 _("Failed to contact configuration server; some possible causes are that you need to enable TCP/IP networking for ORBit, or you have stale NFS locks due to a system crash. See http://www.gnome.org/projects/gconf/ for information. (Details - %s)"),
+                 failure_log->len > 0 ? failure_log->str : _("none"));
 
+  g_string_free (failure_log, TRUE);
+  
   close (p[0]);
   close (p[1]);
   
   return server;
 }
+
+gboolean
+gconf_CORBA_Object_equal (gconstpointer a, gconstpointer b)
+{
+  CORBA_Environment ev;
+  CORBA_Object _obj_a = (gpointer)a;
+  CORBA_Object _obj_b = (gpointer)b;
+  gboolean retval;
+
+  CORBA_exception_init (&ev);
+  retval = CORBA_Object_is_equivalent(_obj_a, _obj_b, &ev);
+  CORBA_exception_free (&ev);
+
+  return retval;
+}
+
+guint
+gconf_CORBA_Object_hash (gconstpointer key)
+{
+  CORBA_Environment ev;
+  CORBA_Object _obj = (gpointer)key;
+  CORBA_unsigned_long retval;
+
+  CORBA_exception_init (&ev);
+  retval = CORBA_Object_hash(_obj, G_MAXUINT, &ev);
+  CORBA_exception_free (&ev);
+
+  return retval;
+}
+
+void
+_gconf_init_i18n (void)
+{
+  static gboolean done = FALSE;
+
+  if (!done)
+    {
+      bindtextdomain (GETTEXT_PACKAGE, GCONF_LOCALE_DIR);
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+      bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif
+      done = TRUE;
+    }
+}
