@@ -52,6 +52,7 @@
 #include <syslog.h>
 #include <time.h>
 #include <sys/wait.h>
+#include <locale.h>
 
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
@@ -856,13 +857,14 @@ obtain_database (const gchar *address,
   GError* error = NULL;
   GConfDatabase *db;
 
-  g_slist_append(addresses, g_strdup(address));
   db = lookup_database (address);
 
   if (db)
     return db;
-  
+
+  addresses = g_slist_append(addresses, g_strdup(address));
   sources = gconf_sources_new_from_addresses(addresses, &error);
+  g_slist_free (addresses);
 
   if (error != NULL)
     {
@@ -1948,6 +1950,8 @@ logfile_read (void)
   g_slist_free (lines);
   
  finished:
+  if (f != NULL)
+    fclose (f);
   
   g_free (logfile);
   g_free (logdir);
@@ -2300,82 +2304,6 @@ client_count (void)
 static void
 gconf_handle_segv (int signum)
 {
-  static int in_segv = 0;
-  pid_t pid;
-
   return;
-  
-  in_segv++;
-
-  if (in_segv > 2)
-    {
-      /* The fprintf() was segfaulting, we are just totally hosed */
-      _exit(1);
-    }
-  else if (in_segv > 1)
-    {
-      /* dialog display isn't working out */
-      fprintf (stderr, _("Multiple segmentation faults occurred; can't display error dialog\n"));
-      _exit(1);
-    }
-        
-  pid = fork();
-
-  if (pid < 0)
-    {
-      /* Eeeek! Can't show dialog */
-      fprintf (stderr, _("Segmentation fault!\n"
-                         "Cannot display crash dialog: %s\n"),
-               strerror (errno));
-      fflush (stderr);
-      
-      /* Don't use app attributes here - a lot of things are probably hosed */
-      if (g_getenv ("GNOME_DUMP_CORE"))
-        abort ();
-      
-      _exit (1);
-    }
-  else if (pid > 0)
-    {
-      /* Wait for user to see the dialog, then exit. */
-      /* Why wait at all? Because we want to allow people to attach to the
-         process */
-      int estatus;
-      pid_t eret;
-      
-      eret = waitpid (pid, &estatus, 0);
-      
-      /* Don't use app attributes here - a lot of things are probably hosed */
-      if(g_getenv ("GNOME_DUMP_CORE"))
-        abort ();
-      
-      _exit (1);
-    }
-  else /* pid == 0 */
-    {
-      char buf[32];
-      
-      g_snprintf (buf, sizeof (buf), "%d", signum);
-      
-      /* Child process */
-      execl (GCONF_BINDIR "/gnome_segv2", GCONF_BINDIR "/gnome_segv2",
-             GCONFD, buf,
-             VERSION, NULL);
-      
-      execlp ("gnome_segv2", "gnome_segv2",
-              GCONFD, buf,
-              GCONFD, NULL);
-      
-      _exit (99);
-    }
-  
-  in_segv--;
 }
 
-
-
-
-
-
-
-
